/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.base.junit;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.admin.junit.PetalsAdministrationApi;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.CommandRegistration;
import org.ow2.petals.cli.api.command.ConnectionCommand;
import org.ow2.petals.cli.api.command.DisconnectionCommand;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * An abstract class to use by unit test checking that Petals ESB CLI or its extensions embed all its commands, and only
 * the expected commands.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractEmbeddedCommandsTest {

    // We use the Mock implementation for Petals Administration API
    @Rule
    public PetalsAdministrationApi adminApi = new PetalsAdministrationApi();

    @Test
    public void expectedCommands() {

        final Shell myShell = new StringStreamShell();
        CommandRegistration.registers(myShell);

        final List<Class<?>> expectedCommands = new ArrayList<Class<?>>(Arrays.asList(this.getExpectedCommands()));
        final Map<String, Command> commands = myShell.getCommands();
        final Iterator<Command> itCommands = commands.values().iterator();
        while (itCommands.hasNext()) {
            final Command command = itCommands.next();

            final Iterator<Class<?>> itExpectedCommands = expectedCommands.iterator();
            while (itExpectedCommands.hasNext()) {
                final Class<?> expectedCommand = itExpectedCommands.next();
                if (expectedCommand.isInstance(command)) {
                    itExpectedCommands.remove();
                    itCommands.remove();
                    break;
                }
            }
        }

        assertEquals("Few commands are not expected", 0, commands.size());
        assertEquals("Commands are missing", 0, expectedCommands.size());

        this.assertConnectionCommand(myShell.getConnectionCommand());

        this.assertDisconnectionCommand(myShell.getDisconnectionCommand());
    }

    protected abstract Class<?>[] getExpectedCommands();

    protected abstract void assertConnectionCommand(final ConnectionCommand command);

    protected abstract void assertDisconnectionCommand(final DisconnectionCommand command);

}
